@@ -3,7 +3,7 @@ import datetime
 import logging
 
 # third-party imports
-from sqlalchemy import Column, Enum, String, Integer, DateTime, Text
+from sqlalchemy import Column, Enum, String, Integer, DateTime, Text, or_
 from sqlalchemy.ext.hybrid import hybrid_property, hybrid_method
 
 # local imports
@@ -104,6 +104,17 @@ class Otx(SessionBase):
         return q.first()
 
 
+    @staticmethod
+    def account(account_address):
+        session = Otx.create_session()
+        q = session.query(Otx.tx_hash)
+        q = q.join(TxCache)
+        q = q.filter(or_(TxCache.sender==account_address, TxCache.recipient==account_address))
+        txs = q.all()
+        session.close()
+        return list(txs)
+
+
     def __init__(self, nonce, address, tx_hash, signed_tx):
         self.nonce = nonce
         self.tx_hash = tx_hash
@@ -118,68 +129,75 @@ class OtxSync(SessionBase):
     __tablename__ = 'otx_sync'
 
     blockchain = Column(String)
-    height_backlog = Column(Integer)
-    height_session = Column(Integer)
-    height_head = Column(Integer)
+    block_height_backlog = Column(Integer)
+    tx_height_backlog = Column(Integer)
+    block_height_session = Column(Integer)
+    tx_height_session = Column(Integer)
+    block_height_head = Column(Integer)
+    tx_height_head = Column(Integer)
     date_created = Column(DateTime, default=datetime.datetime.utcnow)
     date_updated = Column(DateTime)
 
 
-    @hybrid_method
-    def backlog(self, height=None):
-        session = OtxSync.create_session()
-        if height != None:
-            self.height_backlog = height
-            session.add(self)
-            session.commit()
-        self.date_updated = datetime.datetime.utcnow()
-        height = self.height_backlog
-        session.close()
-        return height
-    
+    def backlog(self, block_height=None, tx_height=None):
+        #session = OtxSync.create_session()
+        if block_height != None:
+            if tx_height == None:
+                raise ValueError('tx height missing')
+            self.block_height_backlog = block_height
+            self.tx_height_backlog = tx_height
+            #session.add(self)
+            self.date_updated = datetime.datetime.utcnow()
+        #session.commit()
+        block_height = self.block_height_backlog
+        tx_height = self.tx_height_backlog
+        #session.close()
+        return (block_height, tx_height)
+   
 
-    @hybrid_method
-    def session(self, height=None):
-        session = OtxSync.create_session()
-        if height != None:
-            self.height_session = height
-            session.add(self)
-            session.commit()
-        self.date_updated = datetime.datetime.utcnow()
-        height = self.height_session
-        session.close()
-        return height
+    def session(self, block_height=None, tx_height=None):
+        #session = OtxSync.create_session()
+        if block_height != None:
+            if tx_height == None:
+                raise ValueError('tx height missing')
+            self.block_height_session = block_height
+            self.tx_height_session = tx_height
+            #session.add(self)
+            self.date_updated = datetime.datetime.utcnow()
+        #session.commit()
+        block_height = self.block_height_session
+        tx_height = self.tx_height_session
+        #session.close()
+        return (block_height, tx_height)
 
 
-    @hybrid_method
-    def head(self, height=None):
-        session = OtxSync.create_session()
-        if height != None:
-            self.height_head = height
-            session.add(self)
-            session.commit()
-        self.date_updated = datetime.datetime.utcnow()
-        height = self.height_head
-        session.close()
-        return height
+    def head(self, block_height=None, tx_height=None):
+        #session = OtxSync.create_session()
+        if block_height != None:
+            if tx_height == None:
+                raise ValueError('tx height missing')
+            self.block_height_head = block_height
+            self.tx_height_head = tx_height
+            #session.add(self)
+            self.date_updated = datetime.datetime.utcnow()
+        #session.commit()
+        block_height = self.block_height_head
+        tx_height = self.tx_height_head
+        #session.close()
+        return (block_height, tx_height)
 
 
     @hybrid_property
     def synced(self):
-        return self.height_session == self.height_backlog
+        #return self.block_height_session == self.block_height_backlog and self.tx_height_session == self.block_height_backlog
+        return self.block_height_session == self.block_height_backlog and self.tx_height_session == self.tx_height_backlog
 
 
     @staticmethod
-    def load(blockchain):
-        session = SessionBase.create_session()
-        s = session.query(OtxSync).filter(OtxSync.blockchain==blockchain).first()
-        if s == None:
-            logg.info('creating new blockchain sync record for {}'.format(blockchain))
-            s = OtxSync(blockchain)
-            session.add(s)
-            session.commit()
-        session.close()
-        return s
+    def load(blockchain_string, session):
+        q = session.query(OtxSync)
+        q = q.filter(OtxSync.blockchain==blockchain_string)
+        return q.first()
 
 
     @staticmethod
@@ -204,8 +222,15 @@ class OtxSync(SessionBase):
         return otxs
 
 
+    def chain(self):
+        return self.blockchain
+
+
     def __init__(self, blockchain):
         self.blockchain = blockchain
-        self.height_head = 0
-        self.height_session = 0
-        self.height_backlog = 0
+        self.block_height_head = 0
+        self.tx_height_head = 0
+        self.block_height_session = 0
+        self.tx_height_session = 0
+        self.block_height_backlog = 0
+        self.tx_height_backlog = 0
