@@ -58,7 +58,7 @@ class TxCache(SessionBase):
         # not automatically set in sqlite, it seems:
         self.date_created = datetime.datetime.now()
         self.date_updated = self.date_created
-        self.date_checked = self.date_checked
+        self.date_checked = self.date_created
 
 
     @staticmethod
