@@ -23,8 +23,7 @@ def sign_tx(tx):
     return (tx_hash_hex, tx_transfer_signed['raw'],)
 
 
-@celery_app.task()
-def sign_and_register_tx(tx):
+def sign_and_register_tx(tx, queue, cache_task=None):
     """Signs the provided transaction, and adds it to the transaction queue cache (with status PENDING).
 
     :param tx: Standard ethereum transaction data
@@ -32,15 +31,34 @@ def sign_and_register_tx(tx):
     :returns: Tuple; Transaction hash, signed raw transaction data
     :rtype: tuple
     """
-    (tx_hash_hex, tx_transfer_signed_raw) = sign_tx(tx)
-    queue_create(
-        tx['nonce'],
-        tx['from'],
-        tx_hash_hex,
-        tx_transfer_signed_raw,
+    (tx_hash_hex, tx_signed_raw_hex) = sign_tx(tx)
+
+    logg.debug('adding queue tx {}'.format(tx_hash_hex))
+
+    s = celery.signature(
+        'cic_eth.queue.tx.create',
+        [
+            tx['nonce'],
+            tx['from'],
+            tx_hash_hex,
+            tx_signed_raw_hex,
+            ],
+        queue=queue,
         )
-    logg.debug('added queue tx {}'.format(tx_hash_hex))
-    return (tx_hash_hex, tx_transfer_signed_raw,)
+
+    if cache_task != None:
+        logg.debug('adding cache task {} tx {}'.format(cache_task, tx_hash_hex))
+        s_cache = celery.signature(
+                cache_task,
+                [tx_signed_raw_hex],
+                queue=queue,
+                )
+        s.link(s_cache)
+        #s_otx_cache.apply_async()
+
+    s.apply_async()
+
+    return (tx_hash_hex, tx_signed_raw_hex,)
 
 
 def create_check_gas_and_send_task(tx_signed_raws_hex, holder_address, gas, tx_hashes_hex=None, queue=None):
