@@ -27,6 +27,7 @@ logg = logging.getLogger()
 
 contract_function_signatures = {
         'transfer': 'a9059cbb',
+        'approve': '095ea7b3',
         }
 
 
@@ -117,6 +118,19 @@ def unpack_transfer(data):
         }
 
 
+def unpack_approve(data):
+    f = data[2:10]
+    if f != contract_function_signatures['approve']:
+        raise ValueError('Invalid approval data ({})'.format(f))
+
+    d = data[10:]
+    return {
+        'to': web3.Web3.toChecksumAddress('0x' + d[64-40:64]),
+        'amount': int(d[64:], 16)
+        }
+
+
+
 @celery_app.task()
 def balance(tokens, holder_address):
     """Return token balances for a list of tokens for given address
@@ -177,16 +191,59 @@ def transfer(self, tokens, holder_address, receiver_address, value):
     txf = TokenTxFactory(holder_address)
 
     tx_transfer = txf.transfer(t['address'], receiver_address, value)
-    (tx_hash_hex, tx_signed_raw_hex) = sign_and_register_tx(tx_transfer)
+    (tx_hash_hex, tx_signed_raw_hex) = sign_and_register_tx(tx_transfer, queue, cache_task='cic_eth.eth.token.otx_cache_transfer')
     
     gas_budget = tx_transfer['gas'] * tx_transfer['gasPrice']
 
-    s_otx_cache = celery.signature(
-            'cic_eth.eth.token.otx_cache_transfer',
-            [tx_hash_hex, tx_signed_raw_hex],
-            queue=queue,
+    s = create_check_gas_and_send_task(
+             [tx_signed_raw_hex],
+             holder_address,
+             gas_budget,
+             [tx_hash_hex],
+             queue,
             )
-    s_otx_cache.apply_async()
+    s.apply_async()
+    return tx_hash_hex
+
+
+@celery_app.task(bind=True)
+def approve(self, tokens, holder_address, spender_address, value):
+    """Approve ERC20 transfer on behalf of holder address
+
+    First argument is a list of tokens, to enable the task to be chained to the symbol to token address resolver function. However, it accepts only one token as argument.
+
+    :raises TokenCountError: Either none or more then one tokens have been passed as tokens argument
+    
+    :param tokens: Token addresses 
+    :type tokens: list of str, 0x-hex
+    :param holder_address: Token holder address
+    :type holder_address: str, 0x-hex
+    :param receiver_address: Token receiver address
+    :type receiver_address: str, 0x-hex
+    :param value: Amount of token, in 'wei'
+    :type value: int
+    :return: Transaction hash for tranfer operation
+    :rtype: str, 0x-hex
+    """
+    # we only allow one token, one transfer
+    if len(tokens) != 1:
+        raise TokenCountError
+
+    queue = self.request.delivery_info['routing_key']
+
+    # retrieve the token interface
+    t = tokens[0]
+
+    c = RpcClient(holder_address)
+
+    chain_spec = CICRegistry.chain_spec
+    TokenTxFactory.init(c.w3, chain_spec) #, holder_address)
+    txf = TokenTxFactory(holder_address)
+
+    tx_transfer = txf.approve(t['address'], spender_address, value)
+    (tx_hash_hex, tx_signed_raw_hex) = sign_and_register_tx(tx_transfer, queue, cache_task='cic_eth.eth.token.otx_cache_approve')
+    
+    gas_budget = tx_transfer['gas'] * tx_transfer['gasPrice']
 
     s = create_check_gas_and_send_task(
              [tx_signed_raw_hex],
@@ -266,7 +323,7 @@ def otx_cache_transfer(
 
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
     tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
-    (txc, cache_id) = cache_transfer_data(tx_hash_hex, tx, True)
+    (txc, cache_id) = cache_transfer_data(tx_hash_hex, tx)
     return txc
 
 
@@ -274,7 +331,6 @@ def otx_cache_transfer(
 def cache_transfer_data(
     tx_hash_hex,
     tx,
-    outgoing,
         ):
     tx_data = unpack_transfer(tx['data'])
     logg.debug('tx data {}'.format(tx_data))
@@ -295,3 +351,42 @@ def cache_transfer_data(
     cache_id = tx_cache.id
     session.close()
     return (tx_hash_hex, cache_id)
+
+
+@celery_app.task()
+def otx_cache_approve(
+        tx_hash_hex,
+        tx_signed_raw_hex,
+        ):
+
+    tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
+    (txc, cache_id) = cache_approve_data(tx_hash_hex, tx)
+    return txc
+
+
+@celery_app.task()
+def cache_approve_data(
+    tx_hash_hex,
+    tx,
+        ):
+
+    tx_data = unpack_approve(tx['data'])
+    logg.debug('tx data {}'.format(tx_data))
+    logg.debug('tx {}'.format(tx))
+
+    session = SessionBase.create_session()
+    tx_cache = TxCache(
+        tx_hash_hex,
+        tx['from'],
+        tx_data['to'],
+        tx['to'],
+        tx['to'],
+        tx_data['amount'],
+        tx_data['amount'],
+            )
+    session.add(tx_cache)
+    session.commit()
+    cache_id = tx_cache.id
+    session.close()
+    return (tx_hash_hex, cache_id)
