@@ -20,7 +20,7 @@ def unpack_signed_raw_tx(tx_raw_bytes, chain_id):
     vb = v - (chain_id * 2) - 35
     s = b''.join([d[7], d[8], bytes([vb])])
     so = KeyAPI.Signature(signature_bytes=s)
-
+    
     d[6] = chain_id
     d[7] = b''
     d[8] = b''
@@ -40,8 +40,9 @@ def unpack_signed_raw_tx(tx_raw_bytes, chain_id):
         'value': int.from_bytes(d[4], 'big'),
         'data': '0x' + d[5].hex(),
         'v': chain_id,
-        'r': '0x',
-        's': '0x',
+        'r': '0x' + s[:32].hex(),
+        's': '0x' + s[32:64].hex(),
         'chainId': chain_id,
+        'hash': z.hex(),
             }
 
