@@ -8,6 +8,7 @@
 # standard imports
 import logging
 import time
+import datetime
 
 # third-party imports
 import celery
@@ -123,16 +124,17 @@ def set_final_status(tx_hash, block=None, fail=False):
     q = q.filter(Otx.tx_hash==tx_hash)
     o = q.first()
 
-    nonce = o.nonce
-    sender = o.sender
-    otxid = o.otxid
-
     if o == None:
         session.close()
         raise NotLocalTxError('queue does not contain tx hash {}'.format(tx_hash))
 
     session.flush()
 
+    nonce = o.nonce
+    sender = o.sender
+    otxid = o.otxid
+
+
     q = session.query(Otx)
     q = q.filter(Otx.tx_hash==tx_hash)
     o = q.first()
@@ -216,6 +218,10 @@ def get_tx_cache(tx_hash):
     q = q.filter(Otx.tx_hash==tx_hash)
     otx = q.first()
 
+    if otx == None:
+        session.close()
+        raise NotLocalTxError(tx_hash)
+
     session.flush()
 
     q = session.query(TxCache)
@@ -227,6 +233,7 @@ def get_tx_cache(tx_hash):
     tx = {
         'tx_hash': otx.tx_hash,
         'signed_tx': otx.signed_tx,
+        'nonce': otx.nonce,
         'status': StatusEnum(otx.status).name,
         'source_token': txc.source_token_address,
         'destination_token': txc.destination_token_address,
@@ -313,28 +320,39 @@ def get_upcoming_tx(status=StatusEnum.PENDING, recipient=None, since=None):
     if recipient != None:
         q = q.filter(TxCache.recipient==recipient)
 
-    if since != None:
-        q = q.filter(TxCache.date_checked<since)
-
     q = q.group_by(TxCache.sender)
-    logg.debug('qqqq {} {}'.format(q, since))
 
     txs = {}
 
     results = q.all()
     for r in results:
-        # TODO: optimize with subquery, view, clever grouping i am not clever enough to write
         q = session.query(Otx)
         q = q.join(TxCache)
         q = q.filter(TxCache.sender==r.sender)
         q = q.filter(Otx.nonce==r.nonce)
+
+        if since != None:
+            q = q.filter(TxCache.date_checked<since)
+
         o = q.first()
 
+        if o == None:
+            continue
+
         tx_signed_bytes = bytes.fromhex(o.signed_tx[2:])
         tx = unpack_signed_raw_tx(tx_signed_bytes, int(CICRegistry.chain_spec.chain_id()))
         #if recipient == None or tx['to'] == recipient:
             #gas += tx['gas'] * tx['gasPrice']
         txs[o.tx_hash] = o.signed_tx
+        
+        q = session.query(TxCache)
+        q = q.filter(TxCache.otx_id==o.id)
+        o = q.first()
+
+        logg.debug('oooo {}'.format(o))
+        o.date_checked = datetime.datetime.now()
+        session.add(o)
+        session.commit()
 
     session.close()
 
