@@ -1,4 +1,4 @@
-class SyncDone(Exception):
+class LoopDone(Exception):
     """Exception raised when a syncing is complete.
     """
     pass
